@@ -1,35 +1,34 @@
 /*
- * This file is part of the Minicom Communications Program,
- * written by Miquel van Smoorenburg 1991/1992/1993.
+ * dial.c	Functions to dial, retry etc. Als contains the dialing
+ *		directory code, _and_ the famous tu-di-di music.
+ *
+ *		This file is part of the minicom communications package,
+ *		Copyright 1991,1992,1993,1994 Miquel van Smoorenburg.
+ *
+ *		This program is free software; you can redistribute it and/or
+ *		modify it under the terms of the GNU General Public License
+ *		as published by the Free Software Foundation; either version
+ *		2 of the License, or (at your option) any later version.
  */
-#include <sys/types.h>
-#if defined (_POSIX_SOURCE) || defined(_BSD43)
-#  include <unistd.h>
-#  include <stdlib.h>
-#  ifdef _MINIX
-#    undef NULL
+#include "port.h"
+#include "minicom.h"
+
+#if VC_MUSIC
+#  ifdef __linux__
+#    include <sys/ioctl.h>
+#    include <sys/kd.h>
+#    include <sys/time.h>
+#  endif
+#  ifdef _COH42
+#    include <sys/vtkd.h>
 #  endif
 #endif
-#if defined(_BSD43) || defined (_SYSV)
-#  define NOSTREAMS
-#  include <sys/file.h>
-#endif
-#include <string.h>
-#ifdef _MINIX
-#  undef NULL
-#endif
-#include <stdio.h>
-#include <sys/stat.h>
-#ifdef linux
-#  include <sys/ioctl.h>
-#  include <sys/kd.h>
-#  include <sys/time.h>
-#endif
-#include "keyboard.h"
-#include "window.h"
-#include "minicom.h"
-#include "vt100.h"
-#include "configsym.h"
+
+/* Values for the "flags". */
+#define FL_ECHO		0x01	/* Local echo on/off. */
+#define FL_DEL		0x02	/* Backspace or DEL */
+#define FL_TAG		0x80	/* This entry is tagged. */
+#define FL_SAVE		0x0f	/* Which portions of flags to save. */
 
 /* Dialing directory. */
 struct dialent {
@@ -42,7 +41,7 @@ struct dialent {
   char baud[8];
   char parity[2];
   char dialtype;
-  char localecho;
+  char flags; /* Was localecho. */
   char bits[2];
   struct dialent *next;
 };
@@ -62,10 +61,12 @@ struct dver {
 #define dialentno(di, no) ((struct dialent *)((char *)(di) + ((no) * sizeof(struct dialent))))  
 
 static struct dialent *dialents;
+static struct dialent *d_man;
 static int nrents = 1;
 static int newtype;
-/* Access to ".daildir" denied? */
+/* Access to ".dialdir" denied? */
 static int dendd = 0;
+static char *tagged;
 
 /*
  * Functions to talk to the modem.
@@ -169,7 +170,7 @@ void sendbreak()
 WIN *dialwin;
 int dialtime;
 
-#if defined (__linux__) && defined(_SELECT)
+#if VC_MUSIC
 
 /*
  * Play music until key is pressed.
@@ -215,42 +216,41 @@ char *s;
 int rtime;
 {
   int f, x;
+  int ret = 0;
 
   wlocate(dialwin, 1, 5);
-  wprintf(dialwin, "    No connection: %s.\n", s);
+  wprintf(dialwin, "    No connection: %s.      \n", s);
   if (rtime < 0) {
   	wprintf(dialwin, "   Press any key to continue..    ");
 	if (check_io(-1, 0, 10000, NULL, NULL) & 2) 
 		(void) keyboard(KGETKEY, 0);
   	return(0);
   }
-  wprintf(dialwin, "     Retry in %2d seconds         ", rtime);
+  wprintf(dialwin, "     Retry in %2d seconds             ", rtime);
   
   for(f = rtime - 1; f >= 0; f--) {
 	x = check_io(-1, 0, 1000, NULL, NULL);
 	if (x & 2) {
 		/* Key pressed - absorb it. */
-		(void) keyboard(KGETKEY, 0);
-		return(-1);
+		x = keyboard(KGETKEY, 0);
+		if (x != ' ') ret = -1;
+		break;
 	}
   	wlocate(dialwin, 14, 6);
   	wprintf(dialwin, "%2d ", f);
   }
-  sleep(1); /* Allow modem time to hangup if redial time == 0 */
+  if (f < 0) sleep(1); /* Allow modem time to hangup if redial time == 0 */
   wlocate(dialwin, 1, 5);
   wprintf(dialwin, "                              \n");
   wprintf(dialwin, "                        ");
-  return(0);
+  return(ret);
 }
 
 /*
  * Dial a number, and display the name.
  */
-int dial(name, num, keypress, dialtype)
-char *name;
-char *num;
-int keypress;
-int dialtype;
+int dial(d)
+struct dialent *d;
 {
   char *s, *t;
   int f, x, nb, retst = -1;
@@ -266,8 +266,8 @@ int dialtype;
   wcursor(dialwin, CNONE);
 
   wputs(dialwin, "\n");
-  wprintf(dialwin, " Dialing : %s\n", name);
-  wprintf(dialwin, "      At : %s\n", num);
+  wprintf(dialwin, " Dialing : %s\n", d->name);
+  wprintf(dialwin, "      At : %s\n", d->number);
   wredraw(dialwin, 1);
 
   /* Tell keyboard routines we need them. */
@@ -282,6 +282,19 @@ int dialtype;
 MainLoop:
   while(++retries <= maxretries) {
 
+	/* See if we need to try the next tagged entry. */
+	if (retries > 1 && (d->flags & FL_TAG)) {
+		do {
+			d = d->next;
+			if (d == (struct dialent *)NULL) d = dialents;
+		} while(!(d->flags & FL_TAG));
+		wlocate(dialwin, 0, 1);
+		wprintf(dialwin, " Dialing : %s", d->name);
+		wclreol(dialwin);
+		wprintf(dialwin, "\n      At : %s", d->number);
+		wclreol(dialwin);
+	}
+
 	/* Calculate dial time */
 	dialtime = atoi(P_MDIALTIME);
 	if (dialtime == 0) dialtime = 45;
@@ -292,24 +305,24 @@ MainLoop:
 	wlocate(dialwin, 0, 3);
 	wprintf(dialwin, "    Time : %-3d", dialtime);
 	if (maxretries > 1) wprintf(dialwin, "     Attempt #%d", retries);
-	wputs(dialwin, "\n\n\n Press any key to cancel dialing");
+	wputs(dialwin, "\n\n\n Any key to cancel, space to retry.");
 	
 	/* Start the dial */
 	m_flush(portfd);
-	switch(dialtype) {
+	switch(d->dialtype) {
 		case 0:
 			mputs(P_MDIALPRE);
-			mputs(num);
+			mputs(d->number);
 			mputs(P_MDIALSUF);
 			break;
 		case 1:
 			mputs(P_MDIALPRE2);
-			mputs(num);
+			mputs(d->number);
 			mputs(P_MDIALSUF2);
 			break;
 		case 2:
 			mputs(P_MDIALPRE3);
-			mputs(num);
+			mputs(d->number);
 			mputs(P_MDIALSUF3);
 			break;
 	}
@@ -327,7 +340,12 @@ MainLoop:
 	    if (x & 2) {
 		/* Key was pressed - cancelled. */
 		mputs(P_MDIALCAN);
-		(void) keyboard(KGETKEY, 0);
+		/* On space retry. */
+		if (keyboard(KGETKEY, 0) == ' ') {
+			dialfailed("Cancelled", 4);
+			m_flush(portfd);
+			break;
+		}
 		(void) keyboard(KSTOP, 0);
 		wclose(dialwin, 1);
 		return(retst);
@@ -350,7 +368,7 @@ MainLoop:
 		    !strncmp(modbuf, P_MCONNECT, strlen(P_MCONNECT))) {
 			retst = 0;
 			/* Try to do auto-bauding */
-			if (sscanf(s + strlen(P_MCONNECT), "%d", &nb) == 1)
+			if (sscanf(modbuf + strlen(P_MCONNECT), "%d", &nb) == 1)
 				retst = nb;
 
 			/* Try to figure out if this system supports DCD */
@@ -358,10 +376,10 @@ MainLoop:
 			bogus_dcd = 1;
 
 			wlocate(dialwin, 1, 6);
-			if (keypress) {
+			if (d->script[0] == 0) {
 				wputs(dialwin,
 				     "Connected. Press any key to continue");
-#if defined (__linux__) && defined(_SELECT)
+#if VC_MUSIC
 				music();
 #else
 				x = check_io(-1, 0, 0, NULL, NULL);
@@ -371,6 +389,8 @@ MainLoop:
 			}
 			keyboard(KSTOP, 0);
 			wclose(dialwin, 1);
+			/* Print out the connect strings. */
+			wprintf(us, "\r\n%s\r\n", modbuf);
 			dialwin = NIL_WIN;
 			return(retst);
 		}
@@ -402,6 +422,7 @@ MainLoop:
 		wlocate(dialwin, 11, 3);
 		wprintf(dialwin, "%-3d  ", dialtime);
 		if (dialtime <= 0) {
+			mputs(P_MDIALCAN);
 			reason = "Timeout";
 			retst = -1;
 			if (retries < maxretries) {
@@ -441,8 +462,8 @@ static struct dialent *mkstdent()
   d->password[0] = 0;
   d->term = 1;
   d->dialtype = 0;
-  d->localecho = 0;
-  strcpy(d->baud, DEF_BAUD);
+  d->flags = 0;
+  strcpy(d->baud, "Curr");
   strcpy(d->bits, "8");
   strcpy(d->parity, "N");
   d->next = (struct dialent *)0;
@@ -467,6 +488,12 @@ int readdialdir()
   if (didread) return(0);
   didread = 1;
   nrents = 1;
+  tagged = (char *)malloc(1);
+  tagged[0] = 0;
+
+  /* Make the manual dial entry. */
+  d_man = mkstdent();
+  strcpy(d_man->name, "Manually entered number");
 
   /* Construct path */
   sprintf(dfile, "%s/.dialdir", homedir);
@@ -538,7 +565,7 @@ int readdialdir()
 	/* Adjust for version. */
 	if (dial_ver.version == 0) {
 		d->dialtype = 0;
-		d->localecho = 0;
+		d->flags = 0;
 	}
   }
   d->next = (struct dialent *)0;
@@ -556,6 +583,7 @@ static void writedialdir()
   FILE *fp;
   int ret;
   struct dver dial_ver;
+  char oldfl;
 
   /* Make no sense if access denied */
   if (dendd) return;
@@ -587,11 +615,14 @@ static void writedialdir()
 
   /* Write dialing directory */
   while(d) {
+	oldfl = d->flags;
+	d->flags &= FL_SAVE;
   	if (fwrite(d, sizeof(struct dialent), (size_t)1, fp) != 1) {
   		werror("Error writing ~/.dialdir!");
   		fclose(fp);
   		return;
   	}
+	d->flags = oldfl;
   	d = d->next;
   }
   fclose(fp);
@@ -625,23 +656,25 @@ struct dialent *d;
   WIN *w;
   int c;
   
-  w = wopen(5, 5, 75, 15, BDOUBLE, stdattr, MFG, MBG, 0, 0, 1);
-  wprintf(w, " A -   Name               : %s\n", d->name);
-  wprintf(w, " B -   Number             : %s\n", d->number);
-  wprintf(w, " C -   Dial string #      : %d\n", d->dialtype + 1);
-  wprintf(w, " D -   Local echo         : %s\n", yesno(d->localecho));
-  wprintf(w, " E -   Script             : %s\n", d->script);
-  wprintf(w, " F -   Username           : %s\n", d->username);
-  wprintf(w, " G -   Password           : %s\n", d->password);
-  wprintf(w, " H -   Terminal Emulation : %s\n", te[d->term - 1]);
-  wprintf(w, " I -   Line Settings      : %s %s%s1",
+  w = wopen(5, 5, 75, 16, BDOUBLE, stdattr, MFG, MBG, 0, 0, 1);
+  wprintf(w, " A -  Name                : %s\n", d->name);
+  wprintf(w, " B -  Number              : %s\n", d->number);
+  wprintf(w, " C -  Dial string #       : %d\n", d->dialtype + 1);
+  wprintf(w, " D -  Local echo          : %s\n", yesno(d->flags & FL_ECHO));
+  wprintf(w, " E -  Script              : %s\n", d->script);
+  wprintf(w, " F -  Username            : %s\n", d->username);
+  wprintf(w, " G -  Password            : %s\n", d->password);
+  wprintf(w, " H -  Terminal Emulation  : %s\n", te[d->term - 1]);
+  wprintf(w, " I -  Backspace key sends : %s\n",
+	d->flags & FL_DEL ? "Delete" : "Backspace");
+  wprintf(w, " J -   Line Settings      : %s %s%s1",
   	d->baud, d->bits, d->parity);
-  wlocate(w, 4, 11);
+  wlocate(w, 4, 12);
   wputs(w, "Change which setting? ");
   wredraw(w, 1);
 
   while(1) {
-      wlocate(w, 26, 11);
+      wlocate(w, 26, 12);
       c = getch();
       if (c >= 'a') c -= 32;
       switch(c) {
@@ -665,9 +698,9 @@ struct dialent *d;
 		wflush();
 		break;
 	case 'D':
-		d->localecho = !d->localecho;
+		d->flags ^= FL_ECHO;
 		wlocate(w, 28, 3);
-		wprintf(w, "%s", yesno(d->localecho));
+		wprintf(w, "%s", yesno(d->flags & FL_ECHO));
 		wflush();
 		break;
   	case 'E':
@@ -687,9 +720,14 @@ struct dialent *d;
   		wlocate(w, 28, 7);
   		wputs(w, te[d->term - 1]);	
   		break;
-  	case 'I':
+	case 'I':
+		d->flags ^= FL_DEL;
+		wlocate(w, 28, 8);
+		wputs(w, d->flags & FL_DEL ? "Delete   " : "Backspace");
+		break;
+  	case 'J':
   		get_bbp(d->baud, d->bits, d->parity);
-  		wlocate(w, 28, 8);
+  		wlocate(w, 28, 9);
   		wprintf(w, "%s %s%s1  ", d->baud, d->bits, d->parity);
   		break;
   	default:
@@ -699,7 +737,7 @@ struct dialent *d;
 }
 
 static WIN *dsub;
-static char *what =  "  Dial    Add     Edit   Remove ";
+static char *what =  "  Dial    Find    Add     Edit   Remove ";
 static int dprev;
 
 /*
@@ -711,7 +749,7 @@ int k;
   if (k == dprev) return;
 
   if (dprev >= 0) {
-  	wlocate(dsub, 22 + 8*dprev, 0);
+  	wlocate(dsub, 18 + 8*dprev, 0);
 	if (!useattr) {
 		wputs(dsub, " ");
 	} else {
@@ -720,7 +758,7 @@ int k;
 	}
   }
   dprev = k;
-  wlocate(dsub, 22 + 8*k, 0);
+  wlocate(dsub, 18 + 8*k, 0);
   if (!useattr) {
 	wputs(dsub, ">");
   } else {
@@ -729,8 +767,8 @@ int k;
   }
 }
 
-  
-static char *fmt = "\r %2d  %-16.16s %-16.16s%5s %s%s1    %-6.6s %-15.15s\n";
+
+static char *fmt = "\r %2d %c %-16.16s %-16.16s%5s %s%s1    %-6.6s %-15.15s\n";
 
 /*
  * Print the dialing directory. Only draw from "cur" to bottom.
@@ -745,10 +783,11 @@ int top, cur;
   start = cur - top;
   dirflush = 0;
   wlocate(dialw, 0, start + 1);
-  for(f = start; f < 16; f++) {
+  for(f = start; f < dialw->ys - 2; f++) {
   	d = getno(f + top);
   	if (d == (struct dialent *)0) break;
-  	wprintf(dialw, fmt, f+1+top, d->name, d->number,
+  	wprintf(dialw, fmt, f+1+top, (d->flags & FL_TAG) ? 16 : ' ',
+		d->name, d->number,
   		d->baud, d->bits, d->parity, te[d->term - 1], d->script);
   }
   dirflush = 1;
@@ -775,8 +814,11 @@ void dialdir()
   char buf[32];
   int pgud = 0;
   int first = 1;
+  int x1, x2;
+  char *s, dname[64];
 
   dprev = -1;
+  dname[0] = 0;
 
   w = wopen(0, LINES - 1, COLS - 1, LINES - 1, BNONE, A_REVERSE, WHITE, BLACK, 1, 0, 1);
 
@@ -788,27 +830,31 @@ void dialdir()
   }
   wclose(w, 1);
   if (buf[0] != 0) {
-  	(void) dial("Manually entered number", buf, 1, 0);
+	strcpy(d_man->number, buf);
+  	(void) dial(d_man);
   	return;
   }
   
   /* Allright, draw the dialing directory! */
   
   dirflush = 0;
-  dsub = wopen(2, 22, 77, 22, BNONE, A_REVERSE | stdattr, MFG, MBG, 0, 0, 1);
-  w = wopen(3, 2, 76, 19, BSINGLE, stdattr, MFG, MBG, 0, 0, 1);
+  x1 = (COLS / 2) - 37;
+  x2 = (COLS / 2) + 37;
+  dsub = wopen(x1 - 1, LINES - 3, x2 + 1, LINES - 3, BNONE,
+			A_REVERSE | stdattr, MFG, MBG, 0, 0, 1);
+  w = wopen(x1, 2, x2, LINES - 6, BSINGLE, stdattr, MFG, MBG, 0, 0, 1);
   wcursor(w, CNONE);
   wtitle(w, TMID, "Dialing Directory");
   wputs(w, "     Name               Number       Line Format Terminal Script\n");
-  wlocate(dsub, 22, 0);
+  wlocate(dsub, 18, 0);
   wputs(dsub, what);
 
   wsetregion(w, 1, w->ys - 1);
   w->doscroll = 0;
 
   prdir(w, top, top);  
-  wlocate(w, 14, 17);
-  wputs(w, "( Press Escape to exit.. )");
+  wlocate(w, 14, w->ys - 1);
+  wputs(w, "( Escape to exit, Space to tag )");
   dhili(subm);
   dirflush = 1;
   wredraw(dsub, 1);
@@ -833,11 +879,11 @@ again:
   		case K_LT:
   		case 'h':
 			subm--;
-			if (subm < 0) subm = 3;
+			if (subm < 0) subm = 4;
   			break;
   		case K_RT:
   		case 'l':
-  			subm = (subm + 1) % 4;
+  			subm = (subm + 1) % 5;
   			break;
   		case K_PGUP:
   		case '\002': /* Control-B */
@@ -849,6 +895,34 @@ again:
   			pgud = 2;
   			quit = 1;
   			break;	
+		case 'd':    /* Dial. */
+			subm = 0;
+			quit = 1;
+			break;
+		case 'f':    /* Find. */
+			subm = 1;
+			quit = 1;
+			break;
+		case 'a':    /* Add. */
+			subm = 2;
+			quit = 1;
+			break;
+		case 'e':    /* Edit. */
+			subm = 3;
+			quit = 1;
+			break;
+		case 'r':    /* Remove. */
+			subm = 4;
+			quit = 1;
+			break;
+		case ' ':    /* Tag. */
+  			wlocate(w, 4, cur + 1 - top);
+			d->flags ^= FL_TAG;
+			wsetattr(w, A_REVERSE | stdattr);
+			wprintf(w, "%c", d->flags & FL_TAG ? 16 : ' ');
+			wsetattr(w, A_NORMAL | stdattr);
+  			cur += (cur < nrents - 1);
+			break;
   		case '\033':
   		case '\r':
   		case '\n':
@@ -864,9 +938,9 @@ again:
   		top--;
   		prdir(w, top, top);	
 	}
-	if (cur - top > 15) {
+	if (cur - top > w->ys - 3) {
 		top++;
-  		prdir(w, top, top);	
+  		prdir(w, top, top);
 	}
   	if (cur != ocur) wcurbar(w, cur + 1 - top, A_REVERSE | stdattr);
   	ocur = cur;
@@ -883,7 +957,7 @@ again:
   /* Page up or down ? */
   if (pgud == 1) { /* Page up */
   	ocur = top;
-  	top -= 16;
+  	top -= w->ys - 2;
   	if (top < 0) top = 0;
   	cur = top;
   	pgud = 0;
@@ -893,12 +967,14 @@ again:
   }
   if (pgud == 2) { /* Page down */
   	ocur = top;
-  	if (top < nrents - 16) {
-		top += 16;
-		if (top > nrents - 16) top = nrents - 16;
+  	if (top < nrents - w->ys + 2) {
+		top += w->ys - 2;
+		if (top > nrents - w->ys + 2) {
+			top = nrents - w->ys + 2;
+		}
 		cur = top;
 	} else
-		cur = nrents - 1;	
+		cur = nrents - 1;
 	pgud = 0;
 	if (ocur != top) prdir(w, top, cur);
 	ocur = cur;
@@ -910,21 +986,37 @@ again:
   	wclose(w, 1);
   	wclose(dsub, 1);
   	writedialdir();
-  	strcpy(P_BAUDRATE, d->baud);
+
+	/* See if any entries were tagged. */
+	for(d1 = dialents; d1; d1 = d1->next)
+		if (d1->flags & FL_TAG) {
+			d = d1;
+			break;
+		}
+
+	/* Now change settings for this entry. */
+	if (atoi(d->baud) != 0)
+  		strcpy(P_BAUDRATE, d->baud);
   	strcpy(P_PARITY, d->parity);
   	strcpy(P_BITS, d->bits);
-	m_setparms(portfd, P_BAUDRATE, P_PARITY, P_BITS);
+	port_init();
   	mode_status();
   	newtype = d->term;
-	vt_set(-1, -1, NULL, -1, -1, d->localecho, -1);
-	local_echo = d->localecho;
-  	if ((nb = dial(d->name, d->number, d->script[0] ? 0 : 1,
-			d->dialtype)) < 0) return;
+	vt_set(-1, -1, NULL, -1, -1, d->flags & FL_ECHO, -1);
+	local_echo = d->flags & FL_ECHO;
+  	if ((nb = dial(d)) < 0) return;
+
+	/* Get out of tagged list. */
+	d->flags &= ~FL_TAG;
+
+	/* Set backspace key. */
+	keyboard(KSETBS, d->flags & FL_DEL ? 127 : 8);
+	strcpy(P_BACKSPACE, d->flags & FL_DEL ? "DEL" : "BS");
 
   	/* Did we detect a baudrate , and can we set it? */
   	if (P_MAUTOBAUD[0] == 'Y' && nb) {
   		sprintf(P_BAUDRATE, "%d", nb);
-		m_setparms(portfd, P_BAUDRATE, P_PARITY, P_BITS);
+		port_init();
   	} else
   		nb = 0;
   	if (newtype != terminal)
@@ -937,8 +1029,34 @@ again:
 	dial_number = d->number;
   	return;
   }
-  /* Add / insert an entry */
+  /* Find an entry */
   if (subm == 1) {
+	s = input("Find an entry", dname);
+	x1 = 0;
+	for(d = dialents; d; d = d->next, x1++)
+		if (strstr(d->name, s)) break;
+	if (d == NULL) {
+		wbell();
+		goto again;
+	}
+	/* Set current to found entry. */
+	ocur = top;
+	cur = x1;
+	/* Find out if it fits on screen. */
+	if (cur < top || cur >= top + w->ys - 2) {
+		/* No, try to put it in the middle. */
+		top = cur - (w->ys / 2) + 1;
+		if (top < 0) top = 0;
+		if (top > nrents - w->ys + 2) {
+			top = nrents - w->ys + 2;
+		}
+	}
+	if (ocur != top) prdir(w, top, top);
+	ocur = cur;
+  }
+
+  /* Add / insert an entry */
+  if (subm == 2) {
   	d1 = mkstdent();
   	if (d1 == (struct dialent *)0) {
   		wbell();
@@ -951,7 +1069,7 @@ again:
   	d1->next = d2;
   	
   	nrents++;
-  	if (cur - top > 15) {
+  	if (cur - top > w->ys - 3) {
   		top++;
   		prdir(w, top, top);
   	} else {
@@ -960,15 +1078,15 @@ again:
   }
 
   /* Edit an entry */
-  if (subm == 2) {
+  if (subm == 3) {
   	dedit(d);
   	wlocate(w, 0, cur + 1 - top);
-  	wprintf(w, fmt, cur+1, d->name, d->number,
-  		d->baud, d->bits, d->parity, te[d->term - 1], d->script);
+  	wprintf(w, fmt, cur+1, (d->flags & FL_TAG) ? 16 : ' ', d->name,
+	d->number, d->baud, d->bits, d->parity, te[d->term - 1], d->script);
   }
   
   /* Delete an entry from the list */
-  if (subm == 3 && ask("Remove entry?", d_yesno) == 0) {
+  if (subm == 4 && ask("Remove entry?", d_yesno) == 0) {
   	if (nrents == 1) {
   		free((char *)d);
   		d = dialents = mkstdent();
@@ -989,7 +1107,7 @@ again:
   		if (cur == nrents) cur--;
 		prdir(w, top, cur);
   	}
-	if (nrents - top <= 15) {
+	if (nrents - top <= w->ys - 3) {
 		wlocate(w, 0, nrents - top + 1);
 		wclreol(w);
 	}
