@@ -3,24 +3,17 @@
  *		Currently this code is used in _both_ the BBS
  *		system and minicom.
  *
- * Author:	Miquel van Smoorenburg, <miquels@drinkel.nl.mugnet.org>
- * 
+ *		This file is part of the minicom communications package,
+ *		Copyright 1991,1992,1993,1994 Miquel van Smoorenburg.
+ *
+ *		This program is free software; you can redistribute it and/or
+ *		modify it under the terms of the GNU General Public License
+ *		as published by the Free Software Foundation; either version
+ *		2 of the License, or (at your option) any later version.
  */
 
-#include <sys/types.h>
-#if defined(_MINIX) || defined(linux)
-#  include <termcap.h>
-#else
-char *tgetstr(), *tgoto();
-int tgetent(), tputs(), tgetnum(), tgetflag();
-#endif
-#include <string.h>
-#if defined (_POSIX_SOURCE) || defined(_BSD43)
-#  include <stdlib.h>
-#  include <unistd.h>
-#else
-  char *getenv();
-#endif
+#include "port.h"
+#include "window.h"
 #include "charmap.h"
 
 /* Don't want to include all header stuff for three prototypes from sysdep.c */
@@ -34,17 +27,6 @@ int tgetent(), tputs(), tgetnum(), tgetflag();
   void getrowcols();
 #endif
 
-#if 0 /* Should use it in wprintf */
-#ifdef _MINIX
-#  include <stdarg.h>
-#else
-#  include <varargs.h>
-#endif
-#endif
-
-#include <stdio.h>
-#include <signal.h>
-#include "window.h"
 #ifndef BBS
 #include "config.h"
 #endif
@@ -408,6 +390,8 @@ int rel;
   curwin = NIL_WIN;
 #endif
 
+  (void)rel; /* Reserved for future use. */
+
   if ((w = (WIN *)malloc(sizeof(WIN))) == (WIN *)0) return(w);
   
   offs = (border != BNONE);
@@ -825,7 +809,11 @@ int dir;
 #ifdef _SYSV
   	memcpy((char *)dst, (char *)src, len);
 #else
+#  ifdef _BSD43
+  	bcopy((char *)src, (char *)dst, len);
+#  else
   	memmove((char *)dst, (char *)src, len);
+#  endif
 #endif
   } else {
 	/* Now scroll the memory image. */
